/*=========================================================================
===									===
=== FILE	main.c							===
===									===
=== CONTENTS	Main file for xamigo.					===
===									===
=== PART OF	xamigo							===
===									===
=== COPYRIGHT	Copyright (C) 1992 by Neil Bowers			===
===									===
=========================================================================*/

#include <stdio.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Box.h>
#include <X11/Shell.h>
#include "check16.xbm"
#include "Goban.h"
#include "xamigo.h"

#include "goplayutils.h"
#include "amigo.h"

Pixmap	checkmark;


char	*stat_widget_names[] =
{
	"playerLabel","scoreLabel","playerScore","lastMoveLabel","lastMove"
};

Widget		black_stat_widgets[5];
Widget		white_stat_widgets[5];
Widget		toplevel,form,box;
Widget		goban,stats_popup,message;
int		current_handicap, new_handicap;
int		current_level, new_level;
int		ingame,counting;
short		humanColor, amigoColor;
short		new_color;
short		human_passed,amigo_passed;
short		amigo_last_x,amigo_last_y,
		human_last_x,human_last_y;

extern short	playLevel;
extern short	showTrees,showMoveReason,groupInfo;
extern char	*playReason;

extern MenuItem	game_items[],level_items[],handicap_items[],level_items[];


XtActionsRec actions[] =
{
	{"inputStone",	InputStone},
	{"pass",	PassMove}
};


/*--------------------------------------------------------
-- main()						--
--------------------------------------------------------*/
void
main(argc, argv)
	unsigned int	argc;
	char		*argv[];
{
	XtAppContext	app_con;
	int		i;


	toplevel = XtAppInitialize(&app_con, "Xamigo", NULL, 0,
					&argc, argv, NULL, NULL, 0);

	form = XtCreateManagedWidget("form", formWidgetClass, toplevel,
							NULL,0);

	box = XtCreateManagedWidget("buttonBox", boxWidgetClass, form,
							NULL,0);


	/*-- create the menus --*/

	CreateAmigoMenus(box);

	goban = XtVaCreateManagedWidget("goban",gobanWidgetClass,form,
			XtNsensitive,	TRUE,
			XtNcursor,	GbCGrayStone,
			0);

	checkmark = XCreateBitmapFromData(XtDisplay(toplevel),
			RootWindowOfScreen(XtScreen(toplevel)),
			check16_bits, check16_width, check16_height);

	XtAppAddActions(app_con, actions, XtNumber(actions));

	ingame = counting = 0;
	new_handicap = current_handicap = 0;
	new_level = playLevel = 7;
	humanColor = WHITE;
	new_color = amigoColor = BLACK;
	XtVaSetValues(game_items[4].widget,XtNleftBitmap,checkmark,0);
	XtVaSetValues(level_items[6].widget,XtNleftBitmap,checkmark,0);
	XtVaSetValues(handicap_items[0].widget,XtNleftBitmap,checkmark,0);

	CreateStatsPopup();

	XtRealizeWidget(toplevel);
	XtAppMainLoop(app_con);
}


/*----------------------------------------------------------------
-- NewGameProc()						--
--	Handler routine for New Game entry on "Game" menu.	--
--	Clear board, initialise AmiGo, and start a new game.	--
----------------------------------------------------------------*/
void
NewGameProc(widget,call_data,closure)
	Widget		widget;
	XtPointer	closure;
	XtPointer	call_data;
{
	ingame = 0;
	current_handicap = new_handicap;
	playLevel = new_level;
	human_passed = amigo_passed = FALSE;

	amigo_last_x = amigo_last_y = human_last_x = human_last_y = 0;
	amigoColor = new_color;
	humanColor = (amigoColor==BLACK?WHITE:BLACK);
	XtVaSetValues(goban,
		XtNcursor,(humanColor==BLACK ? GbCBlackStone: GbCWhiteStone),
		NULL);

	/*-- clear goban *before* calling goRestart() --*/
	GbClearBoard(goban);
	goRestart(current_handicap);

	XtVaSetValues(black_stat_widgets[4],XtNlabel,"      ",0);
	XtVaSetValues(white_stat_widgets[4],XtNlabel,"      ",0);
	XtVaSetValues(message,XtNstring,"",0);

	ingame = 1;

	if ((amigoColor == BLACK && current_handicap == 0) ||
				(amigoColor == WHITE && current_handicap > 0))
		AmigoMove();
}

/*----------------------------------------------------------------
-- QuitProc()							--
--	Handler routine for the Quit item of the Game menu.	--
--	This should perhaps do a bit more.  Eg, if there is a	--
--	game in progress, request confirmation.			--
----------------------------------------------------------------*/
void
QuitProc(widget,call_data,closure)
	Widget		widget;
	XtPointer	closure;
	XtPointer	call_data;
{
	exit(0);
}

/*----------------------------------------------------------------
-- LevelProc()							--
--	Handler routine for "Level" menu.			--
----------------------------------------------------------------*/
void
LevelProc(widget,call_data,closure)
	Widget		widget;
	XtPointer	closure;
	XtPointer	call_data;
{
	Widget		old;
	int		item_num = (int) call_data;

	if (item_num == new_level - 1)
		return;

	old = level_items[new_level-1].widget;
	XtVaSetValues(old,XtNleftBitmap,None,0);

	new_level = item_num + 1;
	XtVaSetValues(widget,XtNleftBitmap,checkmark,0);
}

/*----------------------------------------------------------------
-- OptionProc()							--
--	Handler routine for `option' entries on the "Game" menu	--
----------------------------------------------------------------*/
void
OptionProc(widget,call_data,closure)
	Widget		widget;
	XtPointer	closure;
	XtPointer	call_data;
{
	int		item_num = (int) call_data;
	short		setting;

	if (item_num < 5)
	{
		XtVaSetValues(game_items[3].widget,XtNleftBitmap,None,0);
		XtVaSetValues(game_items[4].widget,XtNleftBitmap,None,0);
		new_color = (item_num==3?WHITE:BLACK);
		XtVaSetValues(widget,XtNleftBitmap,checkmark,0);
		return;
	}
	switch (item_num)
	{
		case 6:		/* Move Reason */
			setting = (showMoveReason = ! showMoveReason);
			break;

		case 7:		/* show lookahead */
			setting = (showTrees = ! showTrees);
	}
	XtVaSetValues(widget,XtNleftBitmap,(setting?checkmark:None),0);
}

/*----------------------------------------------------------------
-- HandicapProc()						--
--	Handler routine for the "Move" menu.			--
----------------------------------------------------------------*/
void
MoveProc(widget,call_data,closure)
	Widget		widget;
	XtPointer	closure;
	XtPointer	call_data;
{
	short		x,y;
	char		mv[8];

	if (!ingame && !counting)
	{
		XtVaSetValues(message,XtNstring,"Game not in progess.",0);
		XBell(XtDisplay(goban),0);
		return;
	}
	switch((int)call_data)
	{
		case 0:		/* pass */
			HumanPassMove();
			break;

		case 1:		/* resign */
			intrMoveReport(humanColor,"resign","");
			XtVaSetValues(message,XtNstring,"Game over.",0);
			XtVaSetValues(goban,XtNcursor,GbCGrayStone,NULL);
			counting = 0;
			ingame = 0;
			break;
	}
}

/*----------------------------------------------------------------
-- HandicapProc()						--
--	Handler routine for the "Handicap" menu.		--
----------------------------------------------------------------*/
void
HandicapProc(widget,call_data,closure)
	Widget		widget;
	XtPointer	closure;
	XtPointer	call_data;
{
	Widget		old;
	int		item_num = (int) call_data;
	int		newh;

	newh = (item_num==0?0:item_num + 1);
	if (newh == new_handicap)
		return;

	old = handicap_items[(new_handicap?new_handicap-1:0)].widget;
	XtVaSetValues(old,XtNleftBitmap,None,0);

	new_handicap = newh;
	XtVaSetValues(widget,XtNleftBitmap,checkmark,0);
}

/*----------------------------------------------------------------
-- PopupStatsProc()						--
--	Popup the stats window.  Simple bit of code which tries	--
--	to put the popup in a sensible position.  Only handles	--
--	one case right now.					--
----------------------------------------------------------------*/
void
PopupStatsProc(widget,call_data,closure)
	Widget		widget;
	XtPointer	closure;
	XtPointer	call_data;
{
	Dimension	width,height,twidth,theight;
	Position	x,y,tx,ty;


	XtVaGetValues(stats_popup,
		XtNx,&x,
		XtNy,&y,
		XtNwidth,&width,
		XtNheight,&height,
		0);
	XtVaGetValues(toplevel,
		XtNx,&tx,
		XtNy,&ty,
		XtNwidth,&twidth,
		XtNheight,&theight,
		0);
	if (ty > height+40)
		XtVaSetValues(stats_popup,
			XtNx,tx,
			XtNy,ty-height-40,
			0);
	XtPopup(stats_popup,XtGrabNone);
}

/*----------------------------------------------------------------
-- StatDoneProc()						--
--	Callback routine for the "Done" button on the Stats	--
--	popup window.  Pop down the stats window.		--
----------------------------------------------------------------*/
void
StatDoneProc(widget,call_data,closure)
	Widget		widget;
	XtPointer	closure;
	XtPointer	call_data;
{
	XtPopdown(stats_popup);
}

/*----------------------------------------------------------------
-- CreateStatsPopup()						--
--	Create the stats popup window, which displays captured	--
--	stone count, last move, any messages from AmiGo, and	--
--	the final score.					--
----------------------------------------------------------------*/
CreateStatsPopup()
{
	Widget		statform,form,done;
	char		*getenv(), *username;
	Dimension	width;
	int		i;

	stats_popup = XtCreatePopupShell("statsPopup",
				transientShellWidgetClass,toplevel,NULL,0);

	username = getenv("USER");
	statform = XtCreateManagedWidget("statForm", formWidgetClass,
				stats_popup, NULL,0);

	form = XtCreateManagedWidget("blackStatForm",formWidgetClass,statform,
				NULL,0);

	for (i=0;i<5;i++)
		black_stat_widgets[i] = XtCreateManagedWidget(
				stat_widget_names[i],labelWidgetClass,
				form,NULL,0);

	form = XtCreateManagedWidget("whiteStatForm",formWidgetClass,statform,
				NULL,0);

	for (i=0;i<5;i++)
		white_stat_widgets[i] = XtCreateManagedWidget(
				stat_widget_names[i],labelWidgetClass,
				form,NULL,0);

	message = XtCreateManagedWidget("messageWindow",asciiTextWidgetClass,
				statform,NULL,0);

	done = XtCreateManagedWidget("statDone",commandWidgetClass,statform,
				NULL,0);

	XtAddCallback(done,XtNcallback,StatDoneProc,NULL);

	XtRealizeWidget(stats_popup);
}

/*----------------------------------------------------------------
-- HumanPassMove()						--
--	Called when (human) player passes: check for end of	--
--	game.  Should we pass a stone to the opponent?		--
--	Generate a move for computer player.			--
----------------------------------------------------------------*/
void
HumanPassMove()
{
	if (!ingame)
	{
		if (counting)
		{
			CountUp();
			counting=0;
		}
		return;
	}
	intrMoveReport(humanColor,"pass  ","");
	human_passed = TRUE;
	CheckBothPassed();
	if (ingame)
		AmigoMove();
}

/*----------------------------------------------------------------
-- AmigoMove()							--
--	Generate a move for the computer player.  Update	--
--	message window, mark stone placed.  If we passed, check	--
--	to see if game is over.					--
----------------------------------------------------------------*/
AmigoMove()
{
	short	x,y;
	char	mv[8];


	XtVaSetValues(message,XtNstring,"I'm thinking...",0);
	UpdateWidget(message);
	if (genMove(amigoColor,&x,&y))
	{
		sprintf(mv,"%c%d%s",(x<8?x+'A':x+'B'),y+1,(y<9?"    ":"   "));
		GoPlaceStone(amigoColor,x,y);
		if (amigo_last_x != 0)
			GbSetMark(goban,amigo_last_x,amigo_last_y,0,0);
		amigo_last_x = x+1;
		amigo_last_y = y+1;
		GbSetMark(goban,amigo_last_x,amigo_last_y,0,GbMDiamondMark);
		intrMoveReport(amigoColor,mv,playReason);
		amigo_passed = FALSE;
	}
	else
	{
		intrMoveReport(amigoColor,"pass  ","");
		amigo_passed = TRUE;
		CheckBothPassed();
	}
}

/*----------------------------------------------------------------
-- CheckBothPassed()						--
--	Check to see whether the last two moves were both PASS.	--
--	If so, go into `remove dead stone' mode.		--
----------------------------------------------------------------*/
CheckBothPassed()
{
	if (human_passed && amigo_passed)
	{
		XtVaSetValues(message,XtNstring,
			"Click on dead groups.  When all dead groups are removed, select \"pass\" from the Move menu.",0);
		XtVaSetValues(goban,XtNcursor,GbCGrayStone,NULL);
		ingame = 0;
		counting = 1;
	}
}


/*----------------------------------------------------------------
-- PassMove()							--
--	Action proc for the 'p' keyboard command (pass).	--
----------------------------------------------------------------*/
void
PassMove(widget,event)
	Widget		widget;
	XKeyEvent	*event;
{
	HumanPassMove();
}

/*----------------------------------------------------------------
-- InputStone()							--
--	Mouse button press handler for Goban widget.		--
----------------------------------------------------------------*/
void
InputStone(widget,event)
	Widget		widget;
	XButtonEvent	*event;
{
	char		mv[8];
	Position	x	= event->x;
	Position	y	= event->y;


	if (GbGetStoneFromPosition(widget, &x, &y) != TRUE)
		return;

	x--;
	y--;
	sprintf(mv,"%c%d%s",(x<8?x+'A':x+'B'),y+1,(y<9?"    ":"   "));

	if (!ingame && counting)
	{
		DeleteGroupFromStone(x,y);
		return;
	}

	if (!ingame && !counting)
		return;

	if (!GoPlaceStone(humanColor,x,y))
		return;

	if (human_last_x != 0)
		GbSetMark(goban,human_last_x,human_last_y,0,0);
	human_last_x = x+1;
	human_last_y = y+1;
	GbSetMark(goban,human_last_x,human_last_y,0,GbMDiamondMark);

	intrMoveReport(humanColor,mv,"");
	human_passed = FALSE;

	AmigoMove();
}
