/*=========================================================================
===									===
=== FILE	menus.c							===
===									===
=== CONTENTS	Definitions and routines for creating menus.		===
===									===
=== PART OF	xamigo							===
===									===
=== COPYRIGHT	Copyright (C) 1992 by Neil Bowers			===
===									===
=========================================================================*/

#include <stdio.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/Sme.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>
#include "xamigo.h"


MenuItem	game_items[] =
{
	{"newGame",	NewGameProc,	NULL},
	{"stats",	PopupStatsProc,	NULL},
	{"line",	NULL,		NULL},
	{"aWhite",	OptionProc,	NULL},
	{"aBlack",	OptionProc,	NULL},
	{"line",	NULL,		NULL},
	{"reason",	OptionProc,	NULL},
	{"lookahead",	OptionProc,	NULL},
	{"line",	NULL,		NULL},
	{"quit",	QuitProc,	NULL}
};

MenuItem	move_items[] =
{
	{"pass",	MoveProc,	NULL},
	{"resign",	MoveProc,	NULL}
};

MenuItem	level_items[] =
{
	{"level1",	LevelProc,	NULL},
	{"level2",	LevelProc,	NULL},
	{"level3",	LevelProc,	NULL},
	{"level4",	LevelProc,	NULL},
	{"level5",	LevelProc,	NULL},
	{"level6",	LevelProc,	NULL},
	{"level7",	LevelProc,	NULL}
};

MenuItem	handicap_items[] =
{
	{"handicap0",	HandicapProc,	NULL},
	{"handicap2",	HandicapProc,	NULL},
	{"handicap3",	HandicapProc,	NULL},
	{"handicap4",	HandicapProc,	NULL},
	{"handicap5",	HandicapProc,	NULL},
	{"handicap6",	HandicapProc,	NULL},
	{"handicap7",	HandicapProc,	NULL},
	{"handicap8",	HandicapProc,	NULL},
	{"handicap9",	HandicapProc,	NULL}
};


Menu	menus[] =
{
	{"gameMenu", "gameButton",
		game_items, XtNumber(game_items), NULL},

	{"moveMenu", "moveButton",
		move_items, XtNumber(move_items), NULL},

	{"levelMenu", "levelButton",
		level_items, XtNumber(level_items), NULL},

	{"handicapMenu", "handicapButton",
		handicap_items, XtNumber(handicap_items), NULL}
};


void
CreateAmigoMenus(widget)
	Widget	widget;
{
	CreateMenuStrip(widget,menus,XtNumber(menus));
}


void
CreateMenuStrip(container,mdef,num)
	Widget	container;
	Menu	*mdef;
	int	num;
{
	while (num--)
	{
		mdef->menu_button = CreateMenu(container,
			mdef->menu_name,
			mdef->button_name,
			mdef->items,
			mdef->num_items);
		mdef++;
	}
}


Widget
CreateMenu(parent,menu_name,button_name,menu_items,nitems)
	Widget		parent;
	char		*menu_name;
	char		*button_name;
	MenuItem	menu_items[];
	int		nitems;
{
	Widget		button, menu, item;
	int		i;


	button = XtVaCreateManagedWidget(button_name,
			menuButtonWidgetClass, parent,
			XtNmenuName, menu_name,
			0);

	menu = XtCreatePopupShell(menu_name, simpleMenuWidgetClass, button,
							NULL, 0);

	for (i=0;i<nitems;i++)
	{
		if (!strcmp(menu_items[i].item_name,"line"))
			menu_items[i].widget = XtVaCreateManagedWidget(
						menu_items[i].item_name,
						smeLineObjectClass,menu, 0);
		else
		{
			menu_items[i].widget = XtVaCreateManagedWidget(
						menu_items[i].item_name,
						smeBSBObjectClass,menu,
						0);

			XtAddCallback(menu_items[i].widget,
				XtNcallback, menu_items[i].callback,
				(XtPointer)i);
		}
	}

	return button;
}
